"""Copyright 2024 Everlasting Systems and Solutions LLC (www.myeverlasting.net).
All Rights Reserved.

No part of this software or any of its contents may be reproduced, copied, modified or adapted, without the prior written consent of the author, unless otherwise indicated for stand-alone materials.

For permission requests, write to the publisher at the email address below:
office@myeverlasting.net

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
from reportlab.lib.pagesizes import letter,A4
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.platypus import Table, TableStyle
from reportlab.lib.units import inch
from reportlab.graphics.shapes import Drawing, Circle,String
from reportlab.lib import colors
from reportlab.pdfgen import canvas
from io import BytesIO
async def make_pdf_socials(data: dict):
    title = data["title"]
    meta_description = data["meta_description"]
    google_search_desktop = data["google_search_desktop"]
    google_search_mobile = data["google_search_mobile"]
    social_media_tags = data["social_media_tags"]

    # Create a BytesIO object to store the PDF
    pdf_buffer = BytesIO()

    # Create a canvas object for the PDF
    pdf_canvas = canvas.Canvas(pdf_buffer, pagesize=letter)

    # Draw the title
    pdf_canvas.setFont("Helvetica-Bold", 16)
    pdf_canvas.drawString(50, 750, title)

    # Draw the meta description
    pdf_canvas.setFont("Helvetica", 12)
    pdf_canvas.drawString(50, 700, "Meta Description:")
    pdf_canvas.drawString(50, 680, meta_description)

    # Draw the Google Search Results Preview
    pdf_canvas.setFont("Helvetica-Bold", 14)
    pdf_canvas.drawString(50, 630, "Google Search Results Preview:")
    pdf_canvas.setFont("Helvetica", 12)
    pdf_canvas.drawString(50, 610, "Desktop version:")
    pdf_canvas.drawString(50, 590, google_search_desktop)
    pdf_canvas.drawString(50, 570, "Mobile version:")
    pdf_canvas.drawString(50, 550, google_search_mobile)

    # Draw the Social Media Meta Tags
    pdf_canvas.setFont("Helvetica-Bold", 14)
    pdf_canvas.drawString(50, 510, "Social Media Meta Tags:")
    pdf_canvas.setFont("Helvetica", 12)
    pdf_canvas.drawString(50, 490, social_media_tags)

    # Save the PDF and move to the beginning of the buffer
    pdf_canvas.showPage()
    pdf_canvas.save()
    pdf_buffer.seek(0)
    return pdf_buffer


import logging
import boto3
from PIL import Image
from typing import Tuple
from espy_pdfier.util import CONSTANTS
import os
import io


def is_greater_than_allowed_size(image_path):
    if not CONSTANTS.IMAGE_SIZE_MB:
        return False
    return os.path.getsize(image_path) > int(CONSTANTS.IMAGE_SIZE_MB) * 1024 * 1024


def is_image_size_valid(file):
    max_size_mb = float(CONSTANTS.IMAGE_SIZE_MB) * 1024 * 1024
    file_size_mb = file.file.seek(0, 2) / (1024 * 1024)  # Get file size in MB
    file.file.seek(0)  # Reset file pointer to the beginning
    return file_size_mb <= max_size_mb


def resize_image(image_bytes, max_size):
    """
    Resizes an image to the specified max size.

    Args:
        image_bytes: The image data as bytes.
        max_size: The maximum size (width, height) of the resized image.

    Returns:
        A BytesIO object containing the resized image data.
    """
    image = Image.open(io.BytesIO(image_bytes))
    image.thumbnail(max_size)
    resized_image_buffer = io.BytesIO()
    image.save(resized_image_buffer, format=image.format)
    resized_image_buffer.seek(0)
    return resized_image_buffer


def store_image_in_s3(image_buffer, bucket_name, key) -> str:
    """Can be called directly to store image in S3.
    args:
    image_buffer: file.file from FastAPI file upload.
    returns:
    filename: name or key of the image in s3
    """
    try:
        s3 = boto3.client(
            "s3",
            aws_access_key_id=CONSTANTS.ACCESS_KEY,
            aws_secret_access_key=CONSTANTS.SECRET_KEY,
        )
        s3.upload_fileobj(image_buffer, bucket_name, key)
        return f"https://{bucket_name}.s3.amazonaws.com/{key}"
    except Exception as e:
        logging.error(f"An error occured uploadig to s3: {str(e)}")
        raise Exception(f"An error occured: {str(e)}.")


def resize_and_store_images(
    image,
    bucket_name: str,
    thumbnail_size: Tuple[int, int] = (100, 100),
    display_size: Tuple[int, int] = (512, 512),
) -> dict[str, str]:
    """Resizes image (only) to thumbnail and display image while still keeping raw."""
    try:
        if not is_image_size_valid(image):
            raise ValueError("Image size exceeds the allowed limit.")
        name = image.filename
        raw_name = f"raw_{image.filename}"

        store_image_in_s3(image.file, bucket_name, raw_name)

        thumbnail_image = resize_image(image, thumbnail_size)
        display_image = resize_image(image, display_size)

        filename_thumb = f"thumb_{name}"
        store_image_in_s3(thumbnail_image, bucket_name, filename_thumb)

        filename_dp = f"dp_{name}"
        store_image_in_s3(display_image, bucket_name, filename_dp)
        return {
            "thumbnail": f"https://{bucket_name}.s3.amazonaws.com/{filename_thumb}",
            "dp": f"https://{bucket_name}.s3.amazonaws.com/{filename_dp}",
            "raw": f"https://{bucket_name}.s3.amazonaws.com/{raw_name}",
        }

    except Exception as e:
        logging.error(f"An error occurred resizing and storing: {str(e)}")
        raise Exception(f"An error occurred: {str(e)}.")



from setuptools import setup, find_packages

VERSION = "0.0.3"
DESCRIPTION = "ESSL assets management"
LONG_DESCRIPTION = "Internal helper package for ESSL assets management"
setup(
    name="espy_pdfier",
    version="0.10",
    packages=find_packages(),
    install_requires=[
        "reportlab>=4.2.0",
        "pillow>=11.1.0",
        "boto3>=1.36.11",
    ],
    author="Femi Adigun",
    author_email="femi.adigun@myeverlasting.net",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    keywords=["fastapi", "ESSL", "ReachAI"],
)
